#!/usr/bin/env bats
# Run DDL Tensorflow HPM benchmark
readonly NUM_PROCS=2
#Loading the helper file
load test_helper/common_helper

# Parameters (num_processes, script_with_args)
run_mpi() {
    local num_procs=$1
    shift
    eval timeout -s 2 -k 30s 5m ddlrun -v --mpiarg "-pami_noib" --accelerators $num_procs "$@"
}

teardown()
{
    # Cleanup the tfmodel temp directory
    rm -rf /tmp/tfmodel
}

HPM_SCRIPT=${SCRIPTS_DIR}/tf_cnn_benchmarks/tf_cnn_benchmarks.py

DEFAULT_PARAMS="--variable_update=ddl --num_gpus=1 --batch_size=64 --num_batches=200"

# num_gpus are more than 1.The HPM script should always set the num_gpus to 1 when ddl mode is set.
DDL_WITH_NUM_GPUS="--variable_update=ddl --num_gpus=4 --batch_size=64 --num_batches=200"

INCEPTION3_PARAMS="--variable_update=ddl --model=inception3  --num_gpus=1 --batch_size=64 --num_batches=5"

ALEXNET_PARAMS="--variable_update=ddl --model=alexnet  --num_gpus=1 --batch_size=64 --num_batches=5"

RESNET50_PARAMS="--variable_update=ddl --model=resnet50  --num_gpus=1 --batch_size=64 --num_batches=20"

LMS_RESNET50_PARAMS="--lms=True -lms_swapout_threshold 150 --lms_swapin_ahead 1 --lms_swapin_groupby 0 --batch_size=512 --variable_update=ddl --model=resnet50  --num_gpus=1  --num_batches=40"

# This test is intended to test LMS with --image_size but not intended to test the
# absolute max image size.
LMS_GOOGLENET_PARAMS="--lms=True --lms_swapout_threshold=50 --lms_swapin_ahead=6 --lms_swapin_groupby=50 --batch_size=1 --variable_update=ddl --model=googlenet  --num_gpus=1  --num_batches=20 --image_size=6048"


REUSE_TRAIN_DIR_PARAMS="--variable_update=ddl --model=googlenet --batch_size=64 --num_batches=100 --num_gpus=1 --data_name=imagenet --train_dir=training_dir"

run_HPM() {
    mode="${1}"
    shift
    run run_mpi $NUM_PROCS -m ${mode} --ddloption -dump_iter --ddloption 10 --ddloption -dbg_level --ddloption 1 python ${HPM_SCRIPT} "$@"
    assert_success
    assert_output --partial "TensorFlow:"
    assert_output --partial "total images/sec:"
    assert_output --partial "IBM Corp. DDL"
}

@test "Run DDL m mode HPM with default parameters" {
  skip #Skip this test until the m mode is fixed.
  run_HPM m ${DEFAULT_PARAMS}
}

@test "Run DDL r mode HPM with default parameters" {
  run_HPM r ${DEFAULT_PARAMS}
}

@test "Run DDL b mode HPM with default parameters" {
  run_HPM b ${DEFAULT_PARAMS}
}

@test "Run DDL n mode HPM with default parameters" {
  run_HPM n ${DEFAULT_PARAMS}
}

@test "Run DDL b mode HPM with more than one gpu" {
  run run_mpi $NUM_PROCS -m b --ddloption -dump_iter --ddloption 10 --ddloption -dbg_level --ddloption 1 python ${HPM_SCRIPT} ${DDL_WITH_NUM_GPUS}
  assert_failure
  assert_output --partial "DDL benchmarks require num_gpus=1 on each worker"
}

@test "Run DDL d mode HPM with default parameters" {
  skip # mode d is not supported yet.
  run_HPM d ${DEFAULT_PARAMS}
}

@test "Run DDL c mode HPM with default parameters" {
  run_HPM c ${DEFAULT_PARAMS}
}

@test "Run DDL p mode HPM with default parameters" {
  run_HPM p ${DEFAULT_PARAMS}
}

@test "Run DDL p mode HPM with default parameters delayed synchronous" {
  export DDL_SYNC_METHOD=delayed
  run_HPM p ${DEFAULT_PARAMS}
  assert_output --partial "DDL: DDL_SYNC_METHOD=delayed"
}

@test "Run DDL m mode HPM resnet50" {
  skip #Skip this test until the m mode is fixed.
  run_HPM m ${RESNET50_PARAMS}
}

@test "Run DDL r mode HPM resnet50" {
  run_HPM r ${RESNET50_PARAMS}
}

@test "Run DDL b mode HPM resnet50" {
  run_HPM b ${RESNET50_PARAMS}
}

@test "Run DDL n mode HPM resnet50" {
  run_HPM n ${RESNET50_PARAMS}
}

@test "Run DDL d mode HPM resnet50" {
  skip # mode d is not supported yet.
  run_HPM d ${RESNET50_PARAMS}
}

@test "Run DDL c mode HPM resnet50" {
  run_HPM c ${RESNET50_PARAMS}
}

@test "Run DDL p mode HPM resnet50" {
  run_HPM p ${RESNET50_PARAMS}
}

@test "Run DDL p mode HPM resnet50 delayed synchronous" {
  skip # Occasionally fails during cleanup
  export DDL_SYNC_METHOD=delayed
  run_HPM p ${RESNET50_PARAMS}
  assert_output --partial "DDL: DDL_SYNC_METHOD=delayed"
}

@test "Run DDL c mode HPM inception3" {
  run_HPM c ${INCEPTION3_PARAMS}
}

@test "Run DDL c mode HPM alexnet" {
  run_HPM c ${ALEXNET_PARAMS}
}

@test "Run DDL p mode LMS HPM resnet50" {
  run eval timeout -s 2 -k 30s 10m ddlrun -v --mpiarg "-pami_noib" --accelerators $NUM_PROCS "$@" python ${HPM_SCRIPT} ${LMS_RESNET50_PARAMS}
  assert_success
  assert_output --partial "TensorFlow:"
  assert_output --partial "total images/sec:"
  assert_output --partial "IBM Corp. DDL"
  assert_output --partial "Editing model for LMS"
}

@test "Run DDL b mode LMS HPM googlenet" {
  run eval timeout -s 2 -k 30s 10m ddlrun -v -m b --mpiarg "-pami_noib" --accelerators $NUM_PROCS "$@" python ${HPM_SCRIPT} ${LMS_GOOGLENET_PARAMS}
  assert_success
  assert_output --partial "TensorFlow:"
  assert_output --partial "total images/sec:"
  assert_output --partial "IBM Corp. DDL"
  assert_output --partial "Editing model for LMS"
}

@test "Run DDL b mode HPM googlenet with train_dir" {
  # Run twice with the same training directory
  run_HPM b ${REUSE_TRAIN_DIR_PARAMS}
  run_HPM b ${REUSE_TRAIN_DIR_PARAMS}
}
